@@ -1,4 +1,6 @@
 #coding: utf-8
+import logging
+
 from django.http import HttpResponse, Http404
 from django.views.generic.simple import direct_to_template
 from django.shortcuts import get_object_or_404
@@ -13,6 +15,12 @@ from netcash.models import NetcashOrder, NetcashGateway
 from netcash import signals
 from netcash.conf import NETCASH_IP_HEADER
 
+# setup logging
+class NullHandler(logging.Handler):
+    def emit(self, record):
+        pass
+logging.getLogger("netcash").addHandler(NullHandler())
+
 @csrf_exempt
 def data_handler(request, secret):
     """
@@ -24,12 +32,27 @@ def data_handler(request, secret):
     On successful access 'netcash.signals.data' signal is sent.
     Orders should be processed in signal handler.
     """
-    gateway = get_object_or_404(NetcashGateway, secret=secret)
+
+    logger = logging.getLogger("netcash")
+
+    try:
+        gateway = NetcashGateway.objects.get(secret=secret)
+    except NetcashGateway.DoesNotExist:
+        logger.warn('Invalid gateway %s. Request: %s', secret, request.raw_post_data)
+        raise Http404
+
     ip = request.META.get(NETCASH_IP_HEADER, None)
     reference = request.POST.get('Reference', None)
-    order = get_object_or_404(NetcashOrder, pk=reference)
+
+    try:
+        order = NetcashOrder.objects.get(pk=reference)
+    except NetcashOrder.DoesNotExist:
+        logger.warn('Unknown order %s. Request: %s', reference, request.raw_post_data)
+        raise Http404
 
     if order.trusted: # don't process processed orders again
+        logger.warn('Order %s is already processed. Request: %s',
+                    order.pk, request.raw_post_data)
         raise Http404
 
     if gateway.netcash_ip:
